library seached_list;

import 'dart:async';

import 'package:flutter/material.dart';
import 'package:wowit/controllers/base_controller.dart';
import 'package:wowit/controllers/load_more_base_controller.dart';
import 'package:wowit/controllers/searched_controller.dart';
import 'package:wowit/widgets/filter_widget/filter_widget.dart';
import 'package:wowit/widgets/filter_widget/util.dart';
part 'package:wowit/controllers/search_controller.dart';

// ignore: must_be_immutable
class SearchedList extends FilterWidget<_SearchController> {
  SearchedList({
    super.key,
    required this.searchedListModel,
    required super.fieldNameSearching,
    this.searchedDataLoader,
  }) : super(
          _SearchController(searchedDataLoader),
          isFilterOnServer: searchedListModel.isFilterOnServer,
          title: searchedListModel.title,
          filterItems: searchedListModel.filterItems,
        );

  final SearchedListModel searchedListModel;
  final SearchedDataLoader? searchedDataLoader;

  ScrollController scrollController = ScrollController();
  late _SearchController _searchController;
  late Timer timer;

  @override
  Widget getConditionView(Map<String, Object> mapCondition,
      void Function(String fieldName, Object value) onValueChange) {
    return searchedListModel.getConditionView(mapCondition, onValueChange);
  }

  @override
  void loadMore() {
    _searchController.loadMore();
  }

  @override
  Widget getMainView(BuildContext context,
      Map<String, List<Map<String, Object>>> data, BaseController controller) {
    _searchController = controller as _SearchController;

    return _searchController.dataList.isNotEmpty
        ? Padding(
            padding: searchedListPadding ?? const EdgeInsets.all(0.0),
            child: ListView.builder(
              controller: scrollController,
              keyboardDismissBehavior: ScrollViewKeyboardDismissBehavior.onDrag,
              itemCount: _searchController.dataList.length + 1,
              itemBuilder: (BuildContext context, index) {
                if (index < _searchController.dataList.length) {
                  return searchedListModel
                      .getLineView(_searchController.dataList[index]);
                }
                if (_searchController.isLoadMoreRunning == true) {
                  return getLoadingView();
                }
                if (_searchController.hasNextPage == false && index > 20) {
                  return getEndOfPageView();
                }
                return const SizedBox();
              },
            ),
          )
        : getEmptyView(_searchController);
  }

  void search(String keyword) {
    timer.cancel();
    timer = Timer(const Duration(milliseconds: 500), () async {
      onConditionValueChanged(fieldNameSearching, keyword);
      searchedListModel.isFilterOnServer
          ? applyFilterOnServer()
          : applyFilterBaseOnList();
      timer.cancel();
    });
  }
}

class SearchedListModel {
  SearchedListModel({
    required this.title,
    required this.filterItems,
    required this.getLineView,
    required this.getConditionView,
    required this.searchedListPadding,
    required this.isFilterOnServer,
  });

  final String title;
  final List<FilterItem> filterItems;
  final GetLineView getLineView;
  final GetConditionView getConditionView;
  final EdgeInsets searchedListPadding;
  final bool isFilterOnServer;
}
